package cz.mendelu.dnaAnalyser.analyse.cpg;

import cz.mendelu.dnaAnalyser.analyse.zdna.ZdnaAnalyseResult;
import cz.mendelu.dnaAnalyser.utils.analyse.data.DataLocalisedMapper;
import org.apache.ibatis.annotations.*;
import org.apache.ibatis.cursor.Cursor;
import org.apache.ibatis.session.RowBounds;

import java.util.ArrayList;
import java.util.List;

public interface CpgMapper extends DataLocalisedMapper<CpgAnalyseResult> {

    // language=SQL
    String TABLE = "CREATE TABLE IF NOT EXISTS CPG (" +
            "ID integer NOT NULL AUTO_INCREMENT, " +
            "position integer, " +
            "end integer, " +
            "sequence varchar, " +
            "gcPerc float, " +
            "observedToExpectedCpG float, " +
            "length integer, " +
            "PRIMARY KEY (ID))";

    @Select("SELECT count(*) FROM CPG")
    int countAll();

    @Select("SELECT * FROM CPG")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "position"),
            @Result(property = "end", column = "end"),
            @Result(property = "sequence", column = "sequence"),
            @Result(property = "gcPerc", column = "gcPerc"),
            @Result(property = "observedToExpectedCpG", column = "observedToExpectedCpG"),
            @Result(property = "length", column = "length")
    })
    ArrayList<CpgAnalyseResult> getAll();

    @Select("SELECT * FROM CPG WHERE position > #{position} AND end < #{end}")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "position"),
            @Result(property = "end", column = "end"),
            @Result(property = "sequence", column = "sequence"),
            @Result(property = "gcPerc", column = "gcPerc"),
            @Result(property = "observedToExpectedCpG", column = "observedToExpectedCpG"),
            @Result(property = "length", column = "length")
    })
    ArrayList<CpgAnalyseResult> getByWindow(Integer position, Integer end);

    @Select("SELECT * FROM CPG")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "position"),
            @Result(property = "end", column = "end"),
            @Result(property = "sequence", column = "sequence"),
            @Result(property = "gcPerc", column = "gcPerc"),
            @Result(property = "observedToExpectedCpG", column = "observedToExpectedCpG"),
            @Result(property = "length", column = "length")
    })
    Cursor<CpgAnalyseResult> getCursor();

    @Select("SELECT * FROM CPG WHERE ID = #{id}")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "position"),
            @Result(property = "end", column = "end"),
            @Result(property = "sequence", column = "sequence"),
            @Result(property = "gcPerc", column = "gcPerc"),
            @Result(property = "observedToExpectedCpG", column = "observedToExpectedCpG"),
            @Result(property = "length", column = "length")
    })
    CpgAnalyseResult getById(int id);

    @Update("UPDATE CPG SET " +
            "position = #{position}, " +
            "end = #{end}, " +
            "sequence = #{sequence}, " +
            "gcPerc = #{gcPerc}, " +
            "observedToExpectedCpG = #{observedToExpectedCpG} " +
            "length = #{length} " +
            "WHERE ID = #{id}")
    void update(CpgAnalyseResult result);

    @Select("SELECT * FROM CPG WHERE POSITION >= #{startPosition} AND POSITION < #{endPosition} ORDER BY ${orderName} ASC")
    @Results(value = {
            @Result(property = "id", column = "ID"),
            @Result(property = "position", column = "position"),
            @Result(property = "end", column = "end"),
            @Result(property = "sequence", column = "sequence"),
            @Result(property = "gcPerc", column = "gcPerc"),
            @Result(property = "observedToExpectedCpG", column = "observedToExpectedCpG"),
            @Result(property = "length", column = "length")
    })
    List<CpgAnalyseResult> getAllBetweenPositionOrderAsc(
            @Param("startPosition") int startPosition,
            @Param("endPosition") int endPosition,
            @Param("orderName") String orderName,
            RowBounds rowBounds);

    @Delete("DELETE from CPG WHERE ID = #{id}")
    void delete(int id);

    @Insert("INSERT INTO CPG (position, end, sequence, gcPerc, observedToExpectedCpG, length)" +
            " VALUES " +
            "(#{position}, #{end}, #{sequence}, #{gcPerc}, #{observedToExpectedCpG}, #{length})"
    )
    @Options(useGeneratedKeys = true, keyProperty = "id")
    void insert(CpgAnalyseResult result);
}
