package cz.mendelu.dnaAnalyser.analyse.g4hunter.quadruplex;

import com.fasterxml.jackson.annotation.JsonIgnore;
import cz.mendelu.dnaAnalyser.utils.analyse.data.Localised;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Quadruplex place
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
public class AggregateQuadruplex implements Localised {

    private Long id;

    private int position;

    private int length;

    private float score;

    private float absScore;

    private String sequence;

    @JsonIgnore
    private String subScore;

    public AggregateQuadruplex(Quadruplex quadruplex) {
        position = quadruplex.getPosition();
        length = quadruplex.getLength();
        subScore = Float.toString(quadruplex.getScore());
    }

    public List<Float> getSubScoreList() {
        return (StringUtils.isEmpty(subScore))
                ? Collections.EMPTY_LIST
                : Arrays.stream(subScore.split(","))
                    .map(f -> Float.parseFloat(f))
                    .collect(Collectors.toList());
    }

    public void addSubScore(float score) {
        subScore += (StringUtils.isEmpty(subScore))
                ? Float.toString(this.score) + "," + Float.toString(score)
                : "," + Float.toString(score);
    }

    @Override
    public int getMiddle() {
        return this.position + (length/2);
    }
}