package cz.mendelu.dnaAnalyser.sequence.stream;

import cz.mendelu.dnaAnalyser.batch.Batch;
import cz.mendelu.dnaAnalyser.batch.BatchService;
import cz.mendelu.dnaAnalyser.sequence.Sequence;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Optional;
import java.util.UUID;

import static java.text.MessageFormat.format;

@Slf4j
@Service("StreamService")
public class BufferedWindowStreamService implements StreamService {

    @Autowired
    private BatchService batchService;

    private Path storageDir;

    @Value("${sequence.dir}")
    void setStorageDir(String storageDir) {
        this.storageDir = Paths.get(storageDir);
        // Create if not exist
        if (Files.notExists(this.storageDir)) {
            try {
                Files.createDirectories(this.storageDir);
            } catch (IOException e) {
                throw new UnsupportedOperationException(format("Create sequence {0} dir failed.", storageDir), e);
            }
        }
    }

    @Override
    public Optional<WindowStream> open(Sequence sequence, int windowsSize) {
        try {
            Optional<Batch> oBatch = batchService.findBatchFor(sequence);
            if (oBatch.isPresent() == false || oBatch.map(Batch::getStatus).get() != Batch.Status.FINISH) {
                return Optional.empty();
            }

            Path file = getPath(sequence.getBufferId());
            long fileSize = Files.size(file);
            assert fileSize < Integer.MAX_VALUE : "Datafile is to big.";
            assert sequence.getLength() == fileSize : "Sequence size in database and size of file has to by same";

            if (fileSize <= windowsSize) {
                throw new IllegalArgumentException(format("windows size {} is too big, sequence length is only {}.", windowsSize, fileSize));
            }

            return Optional.of(BufferedWindowStream.openFile(file, windowsSize));
        } catch (IOException e) {
            throw new WindowStreamException(format("Sequence file id: {} can't be open", sequence.getBufferId()), e);
        }
    }

    private Path getPath(UUID bufferId) {
        return storageDir.resolve(bufferId.toString());
    }
}
