package cz.mendelu.dnaAnalyser.user;

import com.fasterxml.jackson.annotation.JsonIgnore;
import cz.mendelu.dnaAnalyser.utils.model.Identifiable;
import lombok.*;
import org.hibernate.annotations.Type;
import org.springframework.security.core.userdetails.UserDetails;

import javax.persistence.*;
import java.util.*;

@Data
@Entity
@Builder
@NoArgsConstructor(access = AccessLevel.PROTECTED)
@AllArgsConstructor(access = AccessLevel.PROTECTED)
@Table(name = "users")
public class User implements UserDetails, Identifiable {

    public static User host() {
        Date now = new Date();
        return User.builder()
                .createdDate(now)
                .lastPasswordResetDate(now)
                .authorities(EnumSet.of(Authority.ROLE_HOST))
                .enabled(true)
                .build();
    }

    @Id
    @Builder.Default
    private UUID id = UUID.randomUUID();

    @JsonIgnore
    private String password;

    @Column(unique = true)
    private String username;

    @Temporal(TemporalType.TIMESTAMP)
    private Date lastPasswordResetDate;

    @Temporal(TemporalType.TIMESTAMP)
    private Date createdDate;

    public boolean enabled;

    @ElementCollection(
            targetClass = Authority.class,
            fetch = FetchType.EAGER)
    @CollectionTable(
            name = "USER_AUTORITY",
            joinColumns = @JoinColumn(name = "user_id")
    )
    @Column(name = "authority_id")
    private Set<Authority> authorities;

    @Override
    @JsonIgnore
    public boolean isAccountNonExpired() {
        return enabled;
    }

    @Override
    @JsonIgnore
    public boolean isAccountNonLocked() {
        return enabled;
    }

    @Override
    @JsonIgnore
    public boolean isCredentialsNonExpired() {
        return enabled;
    }
}